import { Signer, providers, ContractReceipt, Contract } from 'ethers';
export { ethers } from 'ethers';

/**
   * Filter the bridges that can perform a specific conversion
   *
   * @param { string } fromToken origin token
   * @param { string } toToken destination token
   * @param { Bridge[] } bridges array of initialized bridges to validate
   *
   * @returns { Bridge[] } the bridges that are capable of convertin fromToken to toToken
   */
declare function getProperBridges(fromToken: string, toToken: string, bridges: Bridge[]): Bridge[];
/** Interface that represents a fee to be paid in the service. This fee is included in the total price */
interface Fee {
    amount: bigint | number;
    decimals: number;
    type: 'Fixed' | 'Percental';
    description: string;
    unit?: string;
}
type Confirmations = Map<bigint, number>;
/** Interface that represents the specification of an operation with a given bridge */
interface BridgeMetadata {
    operation: string;
    maxAmount: bigint;
    minAmount: bigint;
    blocksToDeliver: Confirmations | number;
    fees: Fee[];
}
/** Interface with common operations to know the capabilities of every bridge product */
interface Bridge {
    /**
       * Check if the implementing bridge supports a specific token conversion
       *
       * @param { string } fromToken origin token
       * @param { string } toToken destination token
       *
       * @returns { boolean } if the bridge supports the convertion or not
       */
    supportsConversion: (fromToken: string, toToken: string) => boolean;
    /**
       * Get information about the conditions of the bridge services
       *
       * @returns { BridgeMetadata[] } specification of every possible operation
       */
    getMetadata: () => Promise<BridgeMetadata[]>;
    /**
       * Check if the implementing bridge supports a specific network
       *
       * @param { number } chainId the chain ID
       *
       * @returns { boolean } if the bridge supports the network or not
       */
    supportsNetwork: (chainId: number) => boolean;
}

interface TxResult {
    txHash: string;
    successful: boolean;
}
/** Generalization for the classes that represent a connection to the blockchain */
interface Connection {
    /**
     * Returns current chain height if the connection is working
     *
     * @returns { numner|undefined } current block number
     */
    getChainHeight: () => Promise<number | undefined>;
    /**
     * Get the chain ID of the network that this connection was made for
     *
     * @returns { number } the chain ID
     */
    getChainId: () => Promise<number>;
    /**
     * Get the ethers connection object used as abstraction for the connection
     *
     * @returns { Signer | providers.Provider } ethers abstraction
     */
    getAbstraction: () => Signer | providers.Provider;
    /**
     * Get the receipt for a specific transaction
     *
     * @param { string } tx the transaction hash
     *
     * @returns { Promise<ContractReceipt | null> } promise with the receipt or null
     */
    getTransactionReceipt: (tx: string) => Promise<ContractReceipt | null>;
}
/**
 * Class that represents a connection to the blockchain, this allows to interact with
 * some smart contract functions that the SDK requires
 */
declare class BlockchainConnection implements Connection {
    private readonly _signer;
    private constructor();
    /**
     * Creates the connection object using a EIP-1193 standard. This allows SDK compatibility with wallets
     * like Metamask or any other wallet that injects the window.ethereum object in browser. This method
     * uses ethers.providers.Web3Provider internally to generate the connection.
     *
     * @param { Eip1193Provider } standard EIP-1193 standard object, e.g., window.ethereum
     *
     * @returns { BlockchainConnection } Connection object
     *
     * @throws { FlyoverError } On faliled connection
     */
    static createUsingStandard(standard: providers.ExternalProvider): Promise<BlockchainConnection>;
    /**
     * Creates connection object using encrypted json and its password. This method allows
     * SDK usage from server side. This method uses ethers.providers.JsonRpcProvider internally to
     * generate the connection.
     *
     * @param { any } json encrypted json
     * @param { string } password encrypted json encryption password
     * @param { string } url node url, by default it is RSK testnet public node
     *
     * @returns { BlockchainConnection } Connection object
     *
     * @throws { FlyoverError } On faliled connection
     */
    static createUsingEncryptedJson(json: any, password: string, url?: string): Promise<BlockchainConnection>;
    /**
     * Creates connection object using a passphrase. This method allows SDK usage
     * from server side. This method uses ethers.providers.JsonRpcProvider internally to
     * generate the connection.
     *
     * @param { string } passphrase account passphrase
     * @param { string } url node url, by default it is RSK testnet public node
     *
     * @returns { BlockchainConnection } Connection object
     *
     * @throws { FlyoverError } On faliled connection
     */
    static createUsingPassphrase(passphrase: string, url?: string): Promise<BlockchainConnection>;
    /**
     * Returns authenticated signer address
     *
     * @returns { string } signer address
     */
    getConnectedAddress(): Promise<string>;
    getChainHeight(): Promise<number | undefined>;
    /**
     * Getter for ethers.Signer used object. This object represents a read/write connection to the blockchain
     *
     * @returns { Signer } signer
     */
    get signer(): Signer;
    getChainId(): Promise<number>;
    getAbstraction(): Signer | providers.Provider;
    getTransactionReceipt(tx: string): Promise<ContractReceipt | null>;
    executeTransaction(args: {
        to: string;
        data: string;
        value: string;
    }): Promise<TxResult>;
}
/**
 * Class that represents a readonly connection to the blockchain, this allows to interact with
 * some smart contract views that the SDK requires. All the state-changing functions executed
 * with this class should throw an error
 */
declare class BlockchainReadOnlyConnection implements Connection {
    private readonly _provider;
    constructor(_provider: providers.Provider);
    /**
     * Creates the connection object using a RPC server, it uses  ethers.providers.JsonRpcProvider
     * internally to create the connection
     *
     * @param { string } rpcUrl the url of the JSON RPC server
     *
     * @returns { BlockchainReadOnlyConnection } Connection object
     *
     * @throws { BridgeError } On failed connection
     */
    static createUsingRpc(rpcUrl: string): Promise<BlockchainReadOnlyConnection>;
    getChainHeight(): Promise<number | undefined>;
    getChainId(): Promise<number>;
    getAbstraction(): Signer | providers.Provider;
    getTransactionReceipt(tx: string): Promise<ContractReceipt>;
}
declare function throwErrorIfFailedTx(txResult: TxResult, message?: string): void;
declare function executeContractFunction(contract: Contract, functionName: string, ...params: unknown[]): Promise<TxResult>;
declare function executeContractView<T>(contract: Contract, functionName: string, ...params: unknown[]): Promise<T>;
declare function estimateGas(contract: Contract, functionName: string, ...params: unknown[]): Promise<bigint>;
declare function callContractFunction<R>(contract: Contract, functionName: string, ...params: unknown[]): Promise<R>;

interface HttpClientOptions {
    includeCaptcha: boolean;
}
interface HttpClient {
    get: <T>(url: string, options?: Partial<HttpClientOptions>) => Promise<T>;
    post: <T>(url: string, body: object, options?: Partial<HttpClientOptions>) => Promise<T>;
    getCaptchaToken: CaptchaTokenResolver;
}
type CaptchaTokenResolver = () => Promise<string>;
interface ErrorDetails {
    timestamp: number;
    recoverable: boolean;
    message: string;
    serverUrl?: string;
    details: unknown;
    product?: string;
}
declare class BridgeError extends Error {
    timestamp: number;
    recoverable: boolean;
    details: any;
    serverUrl?: string;
    product?: string;
    constructor(args: ErrorDetails);
}

declare function getHttpClient(resolveCaptchaToken: CaptchaTokenResolver): HttpClient;

/** Interface for the flyover client connection configuration */
interface FlyoverConfig extends BridgesConfig {
    /** Is a custom url for users that want to use other LBC, for example, a local one for testing purposes */
    customLbcAddress?: string;
    /** @deprecated Will be removed in future releases */
    customRegtestUrl?: string;
    /**
     * Function that will be called to get the captcha token returned from a successful captcha challenge
     */
    captchaTokenResolver: CaptchaTokenResolver;
    /**
     * Whether to disable the checksum validation for the RSK addresses or not
     * @default false
     */
    disableChecksum?: boolean;
}

interface TokenBridgeConfig extends BridgesConfig {
    captchaTokenResolver: CaptchaTokenResolver;
    Regtest?: {
        apiURL?: string;
    };
}

/** Available network options */
type Network = 'Mainnet' | 'Testnet' | 'Regtest' | 'Alphanet' | 'Development';
interface BridgesConfig {
    /** Is the name of the network that the client will be using */
    network: Network;
    /**
       * If true http connections will be allowed, otherwise client will throw an error if the connection is not secured
       * @default false
       */
    allowInsecureConnections?: boolean;
    /**
       * Is an object that represnts the connection to the RSK network. It may be obtained using {@link BlockchainConnection.createUsingStandard},
       * {@link BlockchainConnection.createUsingEncryptedJson}, {@link BlockchainReadOnlyConnection.createUsingRpc}} or
       * {@link BlockchainConnection.createUsingPassphrase}. This configuration is optional because it is intended to be used on operations
       * that require direct connection to the Liquidity Bridge Contract from SDK, users who are only interested on operations that use
       * Liquidity Provider Server don't have to provide it
       */
    rskConnection?: Connection;
}

declare const tokens: {
    readonly BTC: "BTC";
    readonly rBTC: "rBTC";
};

declare function deepFreeze<T extends object>(obj: T): T;

declare function decodeBtcAddress(address: string, options?: {
    keepChecksum: boolean;
}): Uint8Array;

declare const BTC_ZERO_ADDRESS_MAINNET = "1111111111111111111114oLvT2";
declare const BTC_ZERO_ADDRESS_TESTNET = "mfWxJ45yp2SFn7UciZyNpvDKrzbhyfKrY8";
declare function validateRequiredFields(objectToValidate: object, ...requiredFields: string[]): void;
declare function isSecureUrl(urlString: string): boolean;
declare function isBtcAddress(address: string): boolean;
declare function isBtcNativeSegwitAddress(address: string): boolean;
declare function isTaprootAddress(address: string): boolean;
declare function isBtcTestnetAddress(address: string): boolean;
declare function isBtcMainnetAddress(address: string): boolean;
declare function isLegacyBtcAddress(address: string): boolean;
declare function isRskAddress(address: string): boolean;
declare function isValidSignature(address: string, message: string, signature: string, isHexMessage?: boolean): boolean;
declare function assertTruthy<T>(value: T | undefined | null, message?: string): asserts value is T;
declare function rskChecksum(address: string, chainId: 30 | 31 | 33): string;
declare function isRskChecksummedAddress(address: string, chainId: 30 | 31 | 33): boolean;
declare function isBtcZeroAddress(config: {
    network: Network;
}, address: string): boolean;

export { BTC_ZERO_ADDRESS_MAINNET, BTC_ZERO_ADDRESS_TESTNET, BlockchainConnection, BlockchainReadOnlyConnection, type Bridge, BridgeError, type BridgeMetadata, type BridgesConfig, type CaptchaTokenResolver, type Confirmations, type Connection, type ErrorDetails, type Fee, type FlyoverConfig, type HttpClient, type HttpClientOptions, type Network, type TokenBridgeConfig, type TxResult, assertTruthy, callContractFunction, decodeBtcAddress, deepFreeze, estimateGas, executeContractFunction, executeContractView, getHttpClient, getProperBridges, isBtcAddress, isBtcMainnetAddress, isBtcNativeSegwitAddress, isBtcTestnetAddress, isBtcZeroAddress, isLegacyBtcAddress, isRskAddress, isRskChecksummedAddress, isSecureUrl, isTaprootAddress, isValidSignature, rskChecksum, throwErrorIfFailedTx, tokens, validateRequiredFields };
