'use strict';

var ethers = require('ethers');
var bech32 = require('bech32');
var bs58check = require('bs58check');
var fetch = require('cross-fetch');
var JSONbig = require('json-bigint');

/**
   * Filter the bridges that can perform a specific conversion
   *
   * @param { string } fromToken origin token
   * @param { string } toToken destination token
   * @param { Bridge[] } bridges array of initialized bridges to validate
   *
   * @returns { Bridge[] } the bridges that are capable of convertin fromToken to toToken
   */
function getProperBridges(fromToken, toToken, bridges) {
    return bridges.filter(bridge => bridge.supportsConversion(fromToken, toToken));
}

/******************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise, SuppressedError, Symbol, Iterator */


function __awaiter(thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
}

typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
};

class BridgeError extends Error {
    constructor(args) {
        super(args.message);
        this.timestamp = args.timestamp;
        this.recoverable = args.recoverable;
        this.details = args.details;
        this.serverUrl = args.serverUrl;
        this.product = args.product;
    }
}

const DEFAULT_NODE_URL = 'https://public-node.testnet.rsk.co';
function connectionError(e) {
    return new BridgeError({
        timestamp: Date.now(),
        recoverable: true,
        message: 'error getting authenticated signer',
        details: {
            error: e.message
        }
    });
}
/**
 * Class that represents a connection to the blockchain, this allows to interact with
 * some smart contract functions that the SDK requires
 */
class BlockchainConnection {
    constructor(_signer) {
        this._signer = _signer;
    }
    /**
     * Creates the connection object using a EIP-1193 standard. This allows SDK compatibility with wallets
     * like Metamask or any other wallet that injects the window.ethereum object in browser. This method
     * uses ethers.providers.Web3Provider internally to generate the connection.
     *
     * @param { Eip1193Provider } standard EIP-1193 standard object, e.g., window.ethereum
     *
     * @returns { BlockchainConnection } Connection object
     *
     * @throws { FlyoverError } On faliled connection
     */
    static createUsingStandard(standard) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const signer = new ethers.providers.Web3Provider(standard).getSigner();
                return new BlockchainConnection(signer);
            }
            catch (e) {
                throw connectionError(e);
            }
        });
    }
    /**
     * Creates connection object using encrypted json and its password. This method allows
     * SDK usage from server side. This method uses ethers.providers.JsonRpcProvider internally to
     * generate the connection.
     *
     * @param { any } json encrypted json
     * @param { string } password encrypted json encryption password
     * @param { string } url node url, by default it is RSK testnet public node
     *
     * @returns { BlockchainConnection } Connection object
     *
     * @throws { FlyoverError } On faliled connection
     */
    static createUsingEncryptedJson(json, password, url = DEFAULT_NODE_URL) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const signer = yield ethers.Wallet.fromEncryptedJson(JSON.stringify(json), password);
                const provider = new ethers.providers.JsonRpcProvider(url);
                yield provider.getNetwork();
                const connectedSigner = signer.connect(provider);
                return new BlockchainConnection(connectedSigner);
            }
            catch (e) {
                throw connectionError(e);
            }
        });
    }
    /**
     * Creates connection object using a passphrase. This method allows SDK usage
     * from server side. This method uses ethers.providers.JsonRpcProvider internally to
     * generate the connection.
     *
     * @param { string } passphrase account passphrase
     * @param { string } url node url, by default it is RSK testnet public node
     *
     * @returns { BlockchainConnection } Connection object
     *
     * @throws { FlyoverError } On faliled connection
     */
    static createUsingPassphrase(passphrase, url = DEFAULT_NODE_URL) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const signer = ethers.Wallet.fromMnemonic(passphrase);
                const provider = new ethers.providers.JsonRpcProvider(url);
                yield provider.getNetwork();
                const connectedSigner = signer.connect(provider);
                return new BlockchainConnection(connectedSigner);
            }
            catch (e) {
                throw connectionError(e);
            }
        });
    }
    /**
     * Returns authenticated signer address
     *
     * @returns { string } signer address
     */
    getConnectedAddress() {
        return __awaiter(this, void 0, void 0, function* () {
            return this.signer.getAddress();
        });
    }
    getChainHeight() {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            return (_a = this.signer.provider) === null || _a === void 0 ? void 0 : _a.getBlockNumber();
        });
    }
    /**
     * Getter for ethers.Signer used object. This object represents a read/write connection to the blockchain
     *
     * @returns { Signer } signer
     */
    get signer() {
        return this._signer;
    }
    getChainId() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._signer.getChainId();
        });
    }
    getAbstraction() {
        return this._signer;
    }
    getTransactionReceipt(tx) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            return (_b = (_a = this._signer.provider) === null || _a === void 0 ? void 0 : _a.getTransactionReceipt(tx)) !== null && _b !== void 0 ? _b : Promise.resolve(null);
        });
    }
    executeTransaction(args) {
        return __awaiter(this, void 0, void 0, function* () {
            const { to, data, value } = args;
            const gasLimit = yield this._signer.estimateGas({ to, data, value });
            const tx = yield this._signer.sendTransaction({ to, data, value, gasLimit });
            const receipt = yield tx.wait();
            return { txHash: receipt.transactionHash, successful: Boolean(receipt.status) };
        });
    }
}
/**
 * Class that represents a readonly connection to the blockchain, this allows to interact with
 * some smart contract views that the SDK requires. All the state-changing functions executed
 * with this class should throw an error
 */
class BlockchainReadOnlyConnection {
    constructor(_provider) {
        this._provider = _provider;
    }
    /**
     * Creates the connection object using a RPC server, it uses  ethers.providers.JsonRpcProvider
     * internally to create the connection
     *
     * @param { string } rpcUrl the url of the JSON RPC server
     *
     * @returns { BlockchainReadOnlyConnection } Connection object
     *
     * @throws { BridgeError } On failed connection
     */
    static createUsingRpc(rpcUrl) {
        return __awaiter(this, void 0, void 0, function* () {
            const provider = new ethers.providers.JsonRpcProvider(rpcUrl);
            return new BlockchainReadOnlyConnection(provider);
        });
    }
    getChainHeight() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._provider.getBlockNumber();
        });
    }
    getChainId() {
        return __awaiter(this, void 0, void 0, function* () {
            return this._provider.getNetwork().then(n => n.chainId);
        });
    }
    getAbstraction() {
        return this._provider;
    }
    getTransactionReceipt(tx) {
        return __awaiter(this, void 0, void 0, function* () {
            return this._provider.getTransactionReceipt(tx);
        });
    }
}
function throwErrorIfFailedTx(txResult, message = 'error executing transaction') {
    if (!txResult.successful) {
        throw new BridgeError({
            timestamp: Date.now(),
            recoverable: true,
            message,
            details: {
                txHash: txResult.txHash
            }
        });
    }
}
function executeContractFunction(contract, functionName, ...params) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const tx = yield contract[functionName](...params);
            const receipt = yield tx.wait();
            return { txHash: receipt.transactionHash, successful: Boolean(receipt.status) };
        }
        catch (e) {
            throw new BridgeError({
                timestamp: Date.now(),
                recoverable: true,
                message: `error executing function ${functionName}`,
                details: {
                    error: e.message
                }
            });
        }
    });
}
function executeContractView(contract, functionName, ...params) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const result = yield contract[functionName](...params);
            return result;
        }
        catch (e) {
            throw new BridgeError({
                timestamp: Date.now(),
                recoverable: true,
                message: `error executing view ${functionName}`,
                details: {
                    error: e.message
                }
            });
        }
    });
}
function estimateGas(contract, functionName, ...params) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const value = yield contract.estimateGas[functionName](...params);
            return value.toBigInt();
        }
        catch (e) {
            throw new BridgeError({
                timestamp: Date.now(),
                recoverable: true,
                message: `error estimating gas for function ${functionName}`,
                details: {
                    error: e.message
                }
            });
        }
    });
}
function callContractFunction(contract, functionName, ...params) {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
            const result = yield contract.callStatic[functionName](...params);
            return result;
        }
        catch (e) {
            throw new BridgeError({
                timestamp: Date.now(),
                recoverable: true,
                message: `error during static call to function ${functionName}`,
                details: {
                    error: e
                }
            });
        }
    });
}

function deepFreeze(obj) {
    Object.entries(obj).forEach((entry) => {
        const key = entry[0];
        const value = entry[1];
        if (typeof value === 'object' && !Object.isFrozen(value)) {
            obj[key] = deepFreeze(value);
        }
    });
    return Object.freeze(obj);
}

const RSK_ADDRESS_REGEX = /^0x[a-fA-F0-9]{40}$/;
// P2TR
const BTC_MAINNET_P2TR_ADDRESS_REGEX = /^bc1p([ac-hj-np-z02-9]{58})$/;
const BTC_TESTNET_P2TR_ADDRESS_REGEX = /^tb1p([ac-hj-np-z02-9]{58})$/;
const BTC_REGTEST_P2TR_ADDRESS_REGEX = /^bcrt1p([ac-hj-np-z02-9]{58})$/;
// P2WSH
const BTC_MAINNET_P2WSH_ADDRESS_REGEX = /^bc1q([ac-hj-np-z02-9]{58})$/;
const BTC_TESTNET_P2WSH_ADDRESS_REGEX = /^tb1q([ac-hj-np-z02-9]{58})$/;
const BTC_REGTEST_P2WSH_ADDRESS_REGEX = /^bcrt1q([ac-hj-np-z02-9]{58})$/;
// P2WPKH
const BTC_MAINNET_P2WPKH_ADDRESS_REGEX = /^bc1q([ac-hj-np-z02-9]{38})$/;
const BTC_TESTNET_P2WPKH_ADDRESS_REGEX = /^tb1q([ac-hj-np-z02-9]{38})$/;
const BTC_REGTEST_P2WPKH_ADDRESS_REGEX = /^bcrt1q([ac-hj-np-z02-9]{38})$/;
// P2SH
const BTC_MAINNET_P2SH_ADDRESS_REGEX = /^[3]([a-km-zA-HJ-NP-Z1-9]{33,34})$/;
const BTC_TESTNET_P2SH_ADDRESS_REGEX = /^[2]([a-km-zA-HJ-NP-Z1-9]{33,34})$/;
// P2PKH (LEGACY)
const BTC_MAINNET_P2PKH_ADDRESS_REGEX = /^[1]([a-km-zA-HJ-NP-Z1-9]{25,34})$/;
const BTC_TESTNET_P2PKH_ADDRESS_REGEX = /^[mn]([a-km-zA-HJ-NP-Z1-9]{25,34})$/;
// Bitcoin zero address as bytes
const BTC_ZERO_ADDRESS_MAINNET = '1111111111111111111114oLvT2';
const BTC_ZERO_ADDRESS_TESTNET = 'mfWxJ45yp2SFn7UciZyNpvDKrzbhyfKrY8';
function validateRequiredFields(objectToValidate, ...requiredFields) {
    const propertiesWithValue = Object.entries(objectToValidate)
        .filter(entry => entry[1] !== null && entry[1] !== undefined)
        .map(entry => entry[0]);
    const missingProperties = requiredFields.filter(prop => !propertiesWithValue.includes(prop));
    if (missingProperties.length !== 0) {
        throw new Error('Validation failed for object with following missing properties: ' + missingProperties.join(', '));
    }
}
function isSecureUrl(urlString) {
    const url = new URL(urlString);
    return url.protocol === 'https:';
}
function isBtcAddress(address) {
    return [
        BTC_MAINNET_P2WSH_ADDRESS_REGEX,
        BTC_TESTNET_P2WSH_ADDRESS_REGEX,
        BTC_REGTEST_P2WSH_ADDRESS_REGEX,
        BTC_MAINNET_P2WPKH_ADDRESS_REGEX,
        BTC_TESTNET_P2WPKH_ADDRESS_REGEX,
        BTC_REGTEST_P2WPKH_ADDRESS_REGEX,
        BTC_MAINNET_P2SH_ADDRESS_REGEX,
        BTC_TESTNET_P2SH_ADDRESS_REGEX,
        BTC_MAINNET_P2PKH_ADDRESS_REGEX,
        BTC_TESTNET_P2PKH_ADDRESS_REGEX,
        BTC_MAINNET_P2TR_ADDRESS_REGEX,
        BTC_TESTNET_P2TR_ADDRESS_REGEX,
        BTC_REGTEST_P2TR_ADDRESS_REGEX
    ].some(regex => regex.test(address));
}
function isBtcNativeSegwitAddress(address) {
    const isNotMixedCase = address === address.toLowerCase() || address === address.toUpperCase();
    return [
        BTC_MAINNET_P2WSH_ADDRESS_REGEX,
        BTC_TESTNET_P2WSH_ADDRESS_REGEX,
        BTC_REGTEST_P2WSH_ADDRESS_REGEX,
        BTC_MAINNET_P2WPKH_ADDRESS_REGEX,
        BTC_TESTNET_P2WPKH_ADDRESS_REGEX,
        BTC_REGTEST_P2WPKH_ADDRESS_REGEX
    ].some(regex => regex.test(address)) && isNotMixedCase;
}
function isTaprootAddress(address) {
    return [
        BTC_MAINNET_P2TR_ADDRESS_REGEX,
        BTC_TESTNET_P2TR_ADDRESS_REGEX,
        BTC_REGTEST_P2TR_ADDRESS_REGEX
    ].some(regex => regex.test(address));
}
function isBtcTestnetAddress(address) {
    return [
        BTC_TESTNET_P2WSH_ADDRESS_REGEX,
        BTC_TESTNET_P2WPKH_ADDRESS_REGEX,
        BTC_TESTNET_P2SH_ADDRESS_REGEX,
        BTC_TESTNET_P2PKH_ADDRESS_REGEX,
        BTC_TESTNET_P2TR_ADDRESS_REGEX,
        BTC_REGTEST_P2TR_ADDRESS_REGEX,
        BTC_REGTEST_P2WSH_ADDRESS_REGEX,
        BTC_REGTEST_P2WPKH_ADDRESS_REGEX
    ].some(regex => regex.test(address));
}
function isBtcMainnetAddress(address) {
    return [
        BTC_MAINNET_P2WSH_ADDRESS_REGEX,
        BTC_MAINNET_P2WPKH_ADDRESS_REGEX,
        BTC_MAINNET_P2SH_ADDRESS_REGEX,
        BTC_MAINNET_P2PKH_ADDRESS_REGEX,
        BTC_MAINNET_P2TR_ADDRESS_REGEX
    ].some(regex => regex.test(address));
}
function isLegacyBtcAddress(address) {
    return [
        BTC_MAINNET_P2PKH_ADDRESS_REGEX,
        BTC_TESTNET_P2PKH_ADDRESS_REGEX,
        BTC_MAINNET_P2SH_ADDRESS_REGEX,
        BTC_TESTNET_P2SH_ADDRESS_REGEX
    ].some(regex => regex.test(address));
}
function isRskAddress(address) {
    return RSK_ADDRESS_REGEX.test(address);
}
function isValidSignature(address, message, signature, isHexMessage = true) {
    message = isHexMessage && !message.startsWith('0x') ? '0x' + message : message;
    const parsedMessage = isHexMessage ? ethers.utils.arrayify(message) : message;
    signature = signature.startsWith('0x') ? signature : '0x' + signature;
    try {
        const recoveredAddress = ethers.utils.verifyMessage(parsedMessage, signature);
        return address === recoveredAddress;
    }
    catch (e) {
        return false;
    }
}
function assertTruthy(value, message = 'unexpected falsy value') {
    if (value === undefined || value === null || value === '' || value === false || value === 0 || (typeof value === 'number' && isNaN(value))) {
        throw new Error(message);
    }
}
function rskChecksum(address, chainId) {
    if (!RSK_ADDRESS_REGEX.test(address)) {
        throw new Error('Invalid RSK address');
    }
    const lowerCaseAddress = address.toLowerCase();
    const hashInput = chainId.toString() + lowerCaseAddress;
    const hash = ethers.utils.keccak256(ethers.utils.toUtf8Bytes(hashInput));
    return lowerCaseAddress.split('')
        .map((character, i) => {
        // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
        return parseInt(hash.at(i), 16) >= 8 ? character.toUpperCase() : character;
    })
        .join('');
}
function isRskChecksummedAddress(address, chainId) {
    return rskChecksum(address, chainId) === address;
}
function isBtcZeroAddress(config, address) {
    if (config.network === 'Mainnet') {
        return address === BTC_ZERO_ADDRESS_MAINNET;
    }
    return address === BTC_ZERO_ADDRESS_TESTNET;
}

function decodeBtcAddress(address, options = { keepChecksum: false }) {
    if (isBtcNativeSegwitAddress(address)) {
        return new Uint8Array(bech32.bech32.decode(address).words);
    }
    else if (isTaprootAddress(address)) {
        return new Uint8Array(bech32.bech32m.decode(address).words);
    }
    else if (isBtcAddress(address)) {
        return options.keepChecksum ? ethers.utils.base58.decode(address) : bs58check.decode(address);
    }
    else {
        throw new Error('not a BTC address');
    }
}

const tokens = deepFreeze({
    BTC: 'BTC',
    rBTC: 'rBTC'
});

const serializer = JSONbig({ useNativeBigInt: true });
function getHttpClient(resolveCaptchaToken) {
    return {
        getCaptchaToken: resolveCaptchaToken,
        get(url, options) {
            return __awaiter(this, void 0, void 0, function* () {
                let request;
                if ((options != null) && options.includeCaptcha === true) {
                    const token = yield this.getCaptchaToken();
                    request = fetch(url, {
                        headers: {
                            'Content-Type': 'application/json',
                            'X-Captcha-Token': token
                        }
                    });
                }
                else {
                    request = fetch(url);
                }
                return request.then((res) => __awaiter(this, void 0, void 0, function* () { return handleResponse(res); }));
            });
        },
        post(url, body, options) {
            return __awaiter(this, void 0, void 0, function* () {
                const headers = {
                    'Content-Type': 'application/json'
                };
                if ((options === null || options === void 0 ? void 0 : options.includeCaptcha) === true) {
                    const token = yield this.getCaptchaToken();
                    headers['X-Captcha-Token'] = token;
                }
                return fetch(url, {
                    method: 'POST', headers, body: serializer.stringify(body)
                }).then((res) => __awaiter(this, void 0, void 0, function* () { return handleResponse(res); }));
            });
        }
    };
}
function handleResponse(response) {
    return __awaiter(this, void 0, void 0, function* () {
        if (response.status >= 400) {
            const responseBody = yield response.json();
            throw new BridgeError({
                serverUrl: response.url,
                message: responseBody.message,
                timestamp: responseBody.timestamp,
                recoverable: responseBody.recoverable,
                details: responseBody.details
            });
        }
        return response.text().then(text => serializer.parse(text));
    });
}

Object.defineProperty(exports, 'ethers', {
  enumerable: true,
  get: function () { return ethers.ethers; }
});
exports.BTC_ZERO_ADDRESS_MAINNET = BTC_ZERO_ADDRESS_MAINNET;
exports.BTC_ZERO_ADDRESS_TESTNET = BTC_ZERO_ADDRESS_TESTNET;
exports.BlockchainConnection = BlockchainConnection;
exports.BlockchainReadOnlyConnection = BlockchainReadOnlyConnection;
exports.BridgeError = BridgeError;
exports.assertTruthy = assertTruthy;
exports.callContractFunction = callContractFunction;
exports.decodeBtcAddress = decodeBtcAddress;
exports.deepFreeze = deepFreeze;
exports.estimateGas = estimateGas;
exports.executeContractFunction = executeContractFunction;
exports.executeContractView = executeContractView;
exports.getHttpClient = getHttpClient;
exports.getProperBridges = getProperBridges;
exports.isBtcAddress = isBtcAddress;
exports.isBtcMainnetAddress = isBtcMainnetAddress;
exports.isBtcNativeSegwitAddress = isBtcNativeSegwitAddress;
exports.isBtcTestnetAddress = isBtcTestnetAddress;
exports.isBtcZeroAddress = isBtcZeroAddress;
exports.isLegacyBtcAddress = isLegacyBtcAddress;
exports.isRskAddress = isRskAddress;
exports.isRskChecksummedAddress = isRskChecksummedAddress;
exports.isSecureUrl = isSecureUrl;
exports.isTaprootAddress = isTaprootAddress;
exports.isValidSignature = isValidSignature;
exports.rskChecksum = rskChecksum;
exports.throwErrorIfFailedTx = throwErrorIfFailedTx;
exports.tokens = tokens;
exports.validateRequiredFields = validateRequiredFields;
